#!/usr/bin/env python
# Package information, generated from cs_package.py.in by make.
#============================================================================
#
#     This file is part of the Code_Saturne Kernel, element of the
#     Code_Saturne CFD tool.
#
#     Copyright (C) 2010-2011 EDF S.A., France
#
#     contact: saturne-support@edf.fr
#
#     The Code_Saturne Kernel is free software; you can redistribute it
#     and/or modify it under the terms of the GNU General Public License
#     as published by the Free Software Foundation; either version 2 of
#     the License, or (at your option) any later version.
#
#     The Code_Saturne Kernel is distributed in the hope that it will be
#     of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.
#
#     You should have received a copy of the GNU General Public License
#     along with the Code_Saturne Kernel; if not, write to the
#     Free Software Foundation, Inc.,
#     51 Franklin St, Fifth Floor,
#     Boston, MA  02110-1301  USA
#
#============================================================================

#-------------------------------------------------------------------------------

import os.path

# Package information
#--------------------

class package:

    def __init__(self):

        self.name = "code_saturne"
        self.tarname = "code_saturne"
        self.version = "2.1.7"
        self.string = "code_saturne 2.1.7"
        self.bugreport = "saturne-support@edf.fr"
        self.url = "http://www.code-saturne.org"

        self.code_name = "Code_Saturne"

        self.preprocessor = "cs_preprocess"
        self.partitioner = "cs_partition"
        self.solver = "cs_solver"
        self.check_syntax = "cs_check_syntax"
        self.io_dump = "cs_io_dump"
        self.guiname = "SaturneGUI"
        self.configfile = "code_saturne.cfg"
        self.scratchdir = 'tmp_Saturne'
        self.srcdir = 'src_saturne'

        self.cc="gcc"
        self.cxx="g++"
        self.fc="gfortran"
        self.ld = "gcc"

        self.cppflags = "-DHAVE_CONFIG_H -I/usr/include   -I/usr/include/mpi -DOMPI_SKIP_MPICXX -I/usr/include/mpi -DOMPI_SKIP_MPICXX -I/usr/include/libxml2 "
        self.cflags="-std=c99 -funsigned-char -pedantic -W -Wall -Wshadow -Wpointer-arith -Wcast-qual -Wcast-align -Wwrite-strings -Wstrict-prototypes -Wmissing-prototypes -Wmissing-declarations -Wnested-externs -Wunused -Wfloat-equal -g -O2 -fstack-protector --param=ssp-buffer-size=4 -Wformat -Wformat-security -Werror=format-security -Wall  -funroll-loops -O2 -Wuninitialized -march=i686"
        self.cxxflags="-ansi -funsigned-char -W -Wall -Wshadow -Wpointer-arith -Wcast-qual -Wcast-align -Wwrite-strings -Wunused -Wno-long-long -g -O2 -fstack-protector --param=ssp-buffer-size=4 -Wformat -Wformat-security -Werror=format-security -Wall  -funroll-loops -O2 -Wuninitialized"
        self.fcflags="-x f95-cpp-input -Wall -Wno-unused  -D_CS_FC_HAVE_FLUSH  -O"
        self.fcmoddir="/usr/include/code_saturne"
        self.fcmodinclude="-I"
        self.ldflags = "      "
        self.libs = "-lsaturne -lmei -lfvm_filters -lfvm -lbft -lple -lcgns -lmedC -lhdf5  -lmpi  -lxml2 -lblas"
        self.deplibs = "-Wl,-export-dynamic -Wl,-Bsymbolic-functions -Wl,-z,relro -O   -lz -ldl -L/usr/lib/gcc/i686-linux-gnu/4.7 -L/usr/lib/gcc/i686-linux-gnu/4.7/../../../i386-linux-gnu -L/usr/lib/gcc/i686-linux-gnu/4.7/../../../../lib -L/lib/i386-linux-gnu -L/lib/../lib -L/usr/lib/i386-linux-gnu -L/usr/lib/../lib -L/usr/lib/gcc/i686-linux-gnu/4.7/../../.. -lgfortran -lm -lquadmath"
        self.rpath="-Wl,-rpath -Wl,"

        # PLE library can be an internal or external library
        # Its flags are included in other build flags,
        # but are made available here for "code_saturne config".
        self.ple_cppflags = "-I/usr/include"
        self.ple_ldflags = ""
        self.ple_libs = "-lple"

        self.prefix = "/usr"
        self.exec_prefix = "/usr"
        self.bindir = "/usr/bin"
        self.includedir = "/usr/include"
        self.pkgincludedir = "/usr/include/code_saturne"
        self.libdir = "/usr/lib"
        self.libexecdir = "/usr/lib"
        self.pkglibexecdir = "/usr/lib/code_saturne"
        self.pythondir = "/usr/lib/python2.7/dist-packages"
        self.pkgpythondir = "/usr/lib/python2.7/dist-packages/code_saturne"
        self.datarootdir = "/usr/share"
        self.datadir = "/usr/share"
        self.pkgdatadir = "/usr/share/code_saturne"
        self.docdir = "/usr/share/doc/code_saturne"
        self.pdfdir = "/usr/share/doc/code_saturne"
        self.sysconfdir = "/etc"
        self.syrthes_prefix = "/usr/lib/syrthes/"

        self.mpi_type = "OpenMPI"
        self.mpi_bindir = ""
        self.mpi_libdir = ""

        self.env_modules = "no"
        self.env_modulecmd = ""

    def get_preprocessor(self):

        preprocessordir = "/usr/lib/code_saturne"
        return os.path.join(preprocessordir, self.preprocessor)

    def get_partitioner(self):

        partitionerdir = "/usr/lib/code_saturne"
        return os.path.join(partitionerdir, self.partitioner)

    def get_solver(self):

        solverdir = "/usr/lib/code_saturne"
        return os.path.join(solverdir, self.solver)

    def get_check_syntax(self):

        checksyntaxdir = "/usr/lib/code_saturne"
        return os.path.join(checksyntaxdir, self.check_syntax)

    def get_io_dump(self):

        iodumpdir = "/usr/lib/code_saturne"
        return os.path.join(iodumpdir, self.io_dump)

    def get_configfile(self):

        configfiledir = "/etc"
        return os.path.join(configfiledir, self.configfile)

    def get_batchdir(self):

        batchdir = "/usr/share/code_saturne"
        return os.path.join(batchdir, 'batch')

    def get_runcase_script(self, script):

        runcasedir = "/usr/share/code_saturne"
        return os.path.join(runcasedir, script)

    def get_alternate_version(self, version):
        """
        Return alternate version package object
        """

        pkg = None

        # Determine path (by absolute or local name)
        pythondir = os.path.normpath(version)
        prefix = os.path.normpath(self.exec_prefix)
        if self.pkgpythondir.find(prefix) > -1:
            postfix = self.pkgpythondir[len(prefix)+1:]
            if not os.path.isabs(pythondir):
                prefix = os.path.split(prefix)[0]
            else:
                prefix = version
            pythondir = os.path.normpath(os.path.join(prefix,
                                                      version,
                                                      postfix))

        # load alternate package
        if os.path.isdir(pythondir):
            import sys
            sys.path.insert(0, pythondir)
            import cs_package
            reload(cs_package)
            pkg = cs_package.package()
            sys.path.pop(0)
        else:
            raise ImportError("Alternative version '" + pythondir + "' not found.")

        return pkg

#-------------------------------------------------------------------------------
